/*
 * Copyright (C) 2012-2013 The Dojo Foundation
 * 
 * Copyright (C) 2012-2013 Rex Young
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.fastmessenger.impl;

class Address {

    private String activeobject, port;
    private int hash = 0;

    Address (String portQName) {
        if (portQName.contains ("::")) {
            throw new IllegalArgumentException ("Validation error: messengerId not supported : "
                    + portQName);
        }

        int position = portQName.indexOf (":");
        if (position == -1) {
            setActiveobject (portQName);
            setPort ("");
        } else {
            setActiveobject (portQName.substring (0, position));
            setPort (portQName.substring (position + 1));
        }
    }

    Address (String activeobject, String port) {
        setActiveobject (activeobject);
        setPort (port);
    }

    void setActiveobject (String activeobject) {
        this.activeobject = activeobject.trim ();
        Utils.chkNonEmptyId (this.activeobject);
    }

    String getActiveobject () {
        return activeobject;
    }

    void setPort (String port) {
        this.port = port.trim ();
        Utils.chkEmptyOkId (this.port);
    }

    String getPort () {
        return port;
    }

    public boolean equals (Object obj) {
        if (this == obj) {
            return true;
        } else if (obj instanceof Address) {
            Address that = (Address) obj;
            return this.activeobject.equals (that.activeobject) && this.port.equals (that.port);
        } else {
            return false;
        }
    }

    public int hashCode () {
        if (hash == 0) {
            hash = toString ().hashCode ();
        }
        return hash;
    }

    public String toString () {
        return activeobject + ":" + port;
    }

}
