/*
 * Copyright (C) 2012-2013 The Dojo Foundation
 * 
 * Copyright (C) 2012-2013 Rex Young
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.fastmessenger.impl;

import com.fastmessenger.model.IReturn;

class Reply<T> implements IReturn<T> {

    private String replyId;
    private Long creationMillis = new Long (System.currentTimeMillis ());

    private Object lockObj = creationMillis;

    private boolean finished = false;
    private Object result = null;

    void setReplyId (String replyId) {
        this.replyId = replyId;
    }

    String getReplyId () {
        return replyId;
    }

    public long getCreationMillis () {
        return creationMillis.longValue ();
    }

    public void setReturnValue (Object returnValue) {
        synchronized (lockObj) {
            if (!finished) {
                result = returnValue;
                finished = true;
                lockObj.notify ();
            }
        }
    }

    public boolean isReturned () {
        synchronized (lockObj) {
            return finished;
        }
    }

    public T get () {
        return (T) get (0);
    }

    @SuppressWarnings("unchecked")
    public T get (long timeoutMillis) {
        long endMillis = 0;
        long waitMillis = 0;

        if (timeoutMillis > 0) {
            endMillis = System.currentTimeMillis () + timeoutMillis;
            waitMillis = timeoutMillis;
        }

        synchronized (lockObj) {
            while (!finished && (timeoutMillis == 0 || waitMillis > 0)) {
                try {
                    lockObj.wait (timeoutMillis == 0 ? 0 : waitMillis);

                    if (!finished && timeoutMillis > 0) {
                        waitMillis = endMillis - System.currentTimeMillis ();
                    }
                } catch (InterruptedException e) {
                }
            }

            if (result instanceof RuntimeException) {
                throw (RuntimeException) result;
            } else if (result instanceof Throwable) {
                throw new RuntimeException ((Throwable) result);
            }

            return (T) result;
        }
    }

}
