/*
 * Copyright (C) 2012-2013 The Dojo Foundation
 * 
 * Copyright (C) 2012-2013 Rex Young
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.fastmessenger.impl;

import java.util.HashMap;
import java.util.UUID;

import com.fastmessenger.model.IReceiver;

class ReplyManager implements IReceiver {

    final Address SelfAddress = new Address (this.getClass ().getName (), "WithReplyId");

    private HashMap<String, Reply<?>> replyMap = new HashMap<String, Reply<?>> ();

    synchronized String addReply (Reply<?> reply) {
        String replyId = null;
        do {
            replyId = UUID.randomUUID ().toString ();
        } while (replyMap.containsKey (replyId));

        replyMap.put (replyId, reply);

        return replyId;
    }

    synchronized Reply<?> removeReply (String replyId) {
        return replyMap.remove (replyId);
    }

    public Object onPortRequest (String port, Object... reqArgs) {
        if (SelfAddress.getPort ().equals (port)) {

            String replyId = (String) reqArgs[0];
            Object returnValue = reqArgs[1];

            Reply<?> reply = removeReply (replyId);

            if (reply != null) {
                reply.setReturnValue (returnValue);
            }
        }
        return null;
    }

}
