/*
 * Copyright (C) 2012-2013 The Dojo Foundation
 * 
 * Copyright (C) 2012-2013 Rex Young
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.fastmessenger.model;

/**
 * This is a marker interface indicating that this is an implementation object for an active object.
 * 
 * @see IMessenger
 * 
 * @author Rex Young
 */
public interface IReceiver {

    /**
     * A messenger object will forward requests placed on all ports of the active object to this
     * method, and guarantees this method a single-threaded runtime environment.
     * 
     * @see IMessenger#registerReceiver(Object, String, String...)
     * @see IMessenger#registerReceiver(Object[], String, String...)
     * 
     * @param port
     *            simple port name
     * @param reqArgs
     *            arguments to invoke the port
     * @return a result value for the request
     */
    Object onPortRequest (String port, Object... reqArgs);

}
