/*
 * Copyright (C) 2012-2013 The Dojo Foundation
 * 
 * Copyright (C) 2012-2013 Rex Young
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.fastmessenger.model;

/**
 * An object that is linked to a request. Through this object a caller has choices either to wait
 * for the result or do something else then come back wait for the result. The caller may also
 * choose to repeatedly query on (non-blocking) whether the result is ready instead of being blocked
 * while waiting.
 * 
 * @see IMessenger
 * @see IMessenger#callService(String, Object...)
 * 
 * @author Rex Young
 * 
 * @param <T>
 *            The type of result value
 */
public interface IReturn<T> {

    /**
     * A non-blocking method that returns true if the request is completed, either normally or with
     * an exception.
     * 
     * @return true if the request is completed
     */
    boolean isReturned ();

    /**
     * A blocking method that Waits if necessary for the request to complete, and then returns the
     * result. It may throw a runtime exception if an exception happens during the execution of the
     * request.
     * 
     * @return a result value
     */
    T get ();

    /**
     * A blocking method that waits if necessary for at most of the given time for the request to
     * complete, and then returns the result, if available. It may throw a runtime exception if an
     * exception happens during the execution of the request.
     * 
     * @param timeoutMillis
     *            the maximum milliseconds to wait
     * @return a result value
     */
    T get (long timeoutMillis);

}
